Imports System
Imports System.Threading
Imports System.Runtime.InteropServices
Imports System.Reflection

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FIT-
' NESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
' iNTERFACEWARE BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
' IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CON-
' NECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

' <summary>
' Simple utility class to catch console control events (ie CTRL-C) in C#. 
' Calls SetConsoleCtrlHandler() in Win32 API.  This is used as part of this example
' to provide a means of stopping the service when it is running in command line mode.
' </summary>
Public Class ConsoleCtrl
    Implements System.IDisposable

    ' <summary>
    ' The event that occurred.
    ' </summary>
    Public Enum ConsoleEvent
        CTRL_C = 0 ' From wincom.h
        CTRL_BREAK = 1
        CTRL_CLOSE = 2
        CTRL_LOGOFF = 5
        CTRL_SHUTDOWN = 6
    End Enum

    ' <summary>
    ' Handler to be called when a console event occurs.
    ' </summary>
    Delegate Sub ControlEventHandler(ByVal consoleEvent As ConsoleEvent)

    ' <summary>
    ' Event fired when a console event occurs
    ' </summary>
    Public Event ControlEvent As ControlEventHandler

    Private eventHandler As ControlEventHandler

    ' <summary>
    ' Create a new instance.
    ' </summary>
    Public Sub New()
        ' save this to a private var so the GC doesn't collect it...
        eventHandler = New ControlEventHandler(AddressOf Handler)
        SetConsoleCtrlHandler(eventHandler, True)
    End Sub 'New


    Protected Overloads Overrides Sub Finalize()
        Dispose(False)
        MyBase.Finalize()
    End Sub 'Finalize


    Public Overloads Sub Dispose() Implements System.IDisposable.Dispose
        Dispose(True)
    End Sub 'Dispose


    Overloads Sub Dispose(ByVal disposing As Boolean)
        If Not (eventHandler Is Nothing) Then
            SetConsoleCtrlHandler(eventHandler, False)
            eventHandler = Nothing
        End If
    End Sub 'Dispose


    Private Sub Handler(ByVal consoleEvent As ConsoleEvent)
        If Not (consoleEvent <> 0) Then
            RaiseEvent ControlEvent(consoleEvent)
        End If
    End Sub 'Handler

    <DllImport("kernel32.dll")> Shared Function SetConsoleCtrlHandler(ByVal e As ControlEventHandler, ByVal add As Boolean) As Boolean

    End Function

End Class


